import json
from typing import List, Dict
from pathlib import Path

def add_snake_to_curve_dict(curves_dict, snake, curve_name, type):

    if type == 'curve':
        out_type = 'curves'
        out_type_node = 'nodes'
    elif type == 'point':
        out_type = 'nodes'
        out_type_node = 'node'
    else:
        raise Exception

    curves_dict[out_type][curve_name] = {}
    curves_dict[out_type][curve_name]['thready_prediction'] = {}
    curves_dict[out_type][curve_name]['thready_prediction']['format'] = 2
    curves_dict[out_type][curve_name]['thready_prediction']['n'] = 1
    curves_dict[out_type][curve_name]['thready_prediction']['vis'] = 'seen'


    if type == 'curve':
        node_keys = []
        for i, node in enumerate(snake):
            temp = {}
            temp['i2'] = 0.01
            temp['x'] = int(node[1])
            temp['y'] = int(node[0])
            temp['z'] = float(node[2])
            node_keys.append(temp)

        curves_dict[out_type][curve_name]['thready_prediction']['instances'] = [{out_type_node: node_keys}]

    if type == 'point':
        node_key = {}
        node_key['i2'] = 0.01
        node_key['x'] = int(snake[0, 1])
        node_key['y'] = int(snake[0, 0])
        node_key['z'] = float(snake[0, 2])

        curves_dict[out_type][curve_name]['thready_prediction']['instances'] = [node_key]

    return curves_dict


def add_snake_to_curve_dict_csv(curves_dict, snake, curve_name):
    curves_dict[curve_name + "_y"] = " ".join([str(value) for value in snake[..., 0]])
    curves_dict[curve_name + "_x"] = " ".join([str(value) for value in snake[..., 1]])

    return curves_dict

def get_keypoint_names_and_colors_from_json(json_path):
    with open(json_path, "r") as read_file:
        data = json.load(read_file)

    keypoint_names = list(data.keys())
    keypoint_cols = []

    for keypoint in keypoint_names:
        r, g, b = data[keypoint]['rgb'].split()
        keypoint_cols.append([float(r), float(g), float(b)])

    return keypoint_names, keypoint_cols


def convert_labels_to_csv(out_labels_dict: dict, labels_to_process: List[str], project: str):

    out_csv_list = []

    for unity_f_code, data in out_labels_dict.items():
        for label in labels_to_process:
            out_csv = {"file": unity_f_code + ".png",
                       "label": label,
                       "user": "scantensus-echo",
                       "time": "",
                       "project": project,
                       "type": data['labels'][label]['type'],
                       "value_y": data['labels'][label]['y'],
                       "value_x": data['labels'][label]['x'],
                       "conf": data['labels'][label]['conf']}

            out_csv_list.append(out_csv)

    return out_csv_list


def convert_labels_to_firebase(out_labels_dict: Dict,
                               labels_to_process: List[str],
                               firebase_reverse_dict: Dict,
                               user_name: str = "thready_prediction"):

    out_firebase_dict = {}

    for unity_code, unity_data in out_labels_dict.items():

        data = unity_data['labels']

        out_unity_code = Path(unity_code).stem + ":png"

        out_firebase_dict[out_unity_code] = {}
        out_firebase_dict[out_unity_code]['curves'] = {}
        out_firebase_dict[out_unity_code]['nodes'] = {}

        for label_name in labels_to_process:
            firebase_label_name = firebase_reverse_dict[label_name]
            path_x = data[label_name]['x']
            path_y = data[label_name]['y']
            path_conf = data[label_name]['conf']
            path_type = data[label_name]['type']

            if path_type == 'curve':
                firebase_type = 'curves'
                firebase_type_node = 'nodes'
            elif path_type == 'point':
                firebase_type = 'nodes'
                firebase_type_node = 'node'
            else: #blurred or off
                continue

            out_firebase_dict[out_unity_code][firebase_type][firebase_label_name] = {}

            working_dict = out_firebase_dict[out_unity_code][firebase_type][firebase_label_name]

            working_dict[user_name] = {}
            working_dict[user_name]['format'] = 2
            working_dict[user_name]['n'] = 1
            working_dict[user_name]['vis'] = 'seen'

            if path_type == 'curve':

                node_keys = []
                xs = path_x.split(" ")
                ys = path_y.split(" ")
                confs = path_conf.split(" ")
                num_nodes = len(xs)

                for i, (x, y, conf) in enumerate(zip(xs, ys, confs)):
                    temp = {}
                    temp['i2'] = 0.01
                    temp['x'] = int(float(x))
                    temp['y'] = int(float(y))
                    temp['z'] = float(conf)
                    if i == 0 and label_name == "curve-lv-endo":
                        temp['nodeKey'] = 'mv-ant-hinge'
                    if i == (num_nodes // 2) and label_name == "curve-lv-endo":
                        temp['nodeKey'] = 'lv-apex-endo'
                    if i == (num_nodes-1) and label_name == "curve-lv-endo":
                        temp['nodeKey'] = 'mv-post-hinge'
                    node_keys.append(temp)

                working_dict[user_name]['instances'] = [{firebase_type_node: node_keys}]

            if path_type == 'point':
                node_key = {}
                node_key['i2'] = 0.01
                node_key['x'] = int(float(path_x))
                node_key['y'] = int(float(path_y))
                node_key['z'] = float(path_conf)
                node_key['nodeKey'] = label_name

                working_dict[user_name]['instances'] = [node_key]

    return out_firebase_dict
